/*
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_vs.h>
#include <sys/byteorder.h>

static uint8_t mfg_data[] = { 0xff, 0xff, 0x00 };

static const struct bt_data ad[] = {
	BT_DATA(BT_DATA_MANUFACTURER_DATA, mfg_data, 3),
};

static void set_tx_power(uint8_t handle_type, uint16_t handle, int8_t tx_pwr_lvl)
{
	struct bt_hci_cp_vs_write_tx_power_level *cp;
	struct bt_hci_rp_vs_write_tx_power_level *rp;
	struct net_buf *buf, *rsp = NULL;
	int err;

	buf = bt_hci_cmd_create(BT_HCI_OP_VS_WRITE_TX_POWER_LEVEL, sizeof(*cp));
	if (!buf) {
		printk("Unable to allocate command buffer\n");
		return;
	}

	cp = net_buf_add(buf, sizeof(*cp));
	cp->handle = sys_cpu_to_le16(handle);
	cp->handle_type = handle_type;
	cp->tx_power_level = tx_pwr_lvl;

	err = bt_hci_cmd_send_sync(BT_HCI_OP_VS_WRITE_TX_POWER_LEVEL, buf, &rsp);
	if (err) {
		uint8_t reason = rsp ?
			((struct bt_hci_rp_vs_write_tx_power_level *) rsp->data)->status : 0;
		printk("Set Tx power err: %d reason 0x%02x\n", err, reason);
		return;
	}

	rp = (void *)rsp->data;
	printk("Actual Tx Power: %d\n", rp->selected_tx_power);

	net_buf_unref(rsp);
}

void main(void)
{
	struct bt_le_ext_adv *adv;
	int err;

	printk("Starting Periodic Advertising Demo\n");

	/* Initialize the Bluetooth Subsystem */
	err = bt_enable(NULL);
	if (err) {
		printk("Bluetooth init failed (err %d)\n", err);
		return;
	}

	struct bt_le_adv_param le_advertising_param = BT_LE_ADV_PARAM_INIT(
		BT_LE_ADV_OPT_EXT_ADV | BT_LE_ADV_OPT_USE_NAME | BT_LE_ADV_OPT_CODED | BT_LE_ADV_OPT_USE_TX_POWER | BT_LE_ADV_OPT_FORCE_NAME_IN_AD,
		0x1f40, 0x2080, NULL);
	
	//ADV Interval range (*0.625ms) 0x0020 (0.02s) to 0x4000 (10.24s)
	// 3s:3.2s (12c0:100)
	// 5s:5.2s (1f40:2080)
	//10s:10.2s (3e80:3fc0)

	/* Create a non-connectable non-scannable advertising set */
	err = bt_le_ext_adv_create(&le_advertising_param, NULL, &adv);
	if (err) {
		printk("Failed to create advertising set (err %d)\n", err);
		return;
	}

	/* Set periodic advertising parameters */
	struct bt_le_per_adv_param periodic_param = BT_LE_PER_ADV_PARAM_INIT(0x1f40, 0x1f40, BT_LE_PER_ADV_OPT_NONE); //for 10s
	//struct bt_le_per_adv_param periodic_param = BT_LE_PER_ADV_PARAM_INIT(0xFA0, 0xFA0, BT_LE_PER_ADV_OPT_NONE); //for 5s
	//struct bt_le_per_adv_param periodic_param = BT_LE_PER_ADV_PARAM_INIT(BT_GAP_PER_ADV_SLOW_INT_MIN, BT_GAP_PER_ADV_SLOW_INT_MAX, BT_LE_PER_ADV_OPT_NONE); //for 1s
	//PERIODIC Interval range (*1.25ms)
	// 10s = 0x1f40
	// 5s = 0xfa0
	// 3s = 0x960
	// 1s = 0x320
	err = bt_le_per_adv_set_param(adv, &periodic_param);
	if (err) {
		printk("Failed to set periodic advertising parameters (err %d)\n", err);
		return;
	}

	set_tx_power(BT_HCI_VS_LL_HANDLE_TYPE_ADV, 0, 0x8);

	/* Enable Periodic Advertising */
	bt_le_per_adv_set_data(adv, ad, ARRAY_SIZE(ad));
	err = bt_le_per_adv_start(adv);
	if (err) {
		printk("Failed to enable periodic advertising (err %d)\n", err);
		return;
	}

	/* Enable Extended Advertising */
	printk("Start Extended Advertising...");
		err = bt_le_ext_adv_start(adv, BT_LE_EXT_ADV_START_DEFAULT);
		if (err) {
			printk("Failed to start extended advertising (err %d)\n", err);
			return;
		}
		printk("done.\n");

	while (true) {
		

		for (int i = 0; i < 3; i++) {
			k_sleep(K_SECONDS(10));

			mfg_data[2]++;

			printk("Set Periodic Advertising Data...");
			err = bt_le_per_adv_set_data(adv, ad, ARRAY_SIZE(ad));
			if (err) {
				printk("Failed (err %d)\n", err);
				return;
			}
			printk("done.\n");
		}

		k_sleep(K_SECONDS(10));
		
	}
}
